#include <WiFi.h>
#include <Wire.h>
#include <BH1750.h>
#include <LiquidCrystal_I2C.h>
#include <Firebase_ESP_Client.h>

// WiFi & Firebase
#define WIFI_SSID "iQOO Z9x 5G"
#define WIFI_PASSWORD "bismillah"
#define API_KEY "AIzaSyB1dN4qCtdNcIkYJAWCBW3HbD-YPlSZ35s"
#define DATABASE_URL "https://fototerphy-default-rtdb.firebaseio.com/"

// Firebase instance
FirebaseData fbdo;
FirebaseAuth auth;
FirebaseConfig config;

// Sensor cahaya
BH1750 lightMeter;

// LCD I2C 16x2 (alamat I2C default 0x27)
LiquidCrystal_I2C lcd(0x27, 16, 2);

// Tombol
#define BUTTON_10MIN 12
#define BUTTON_15MIN 13
#define BUTTON_20MIN 14
#define BUTTON_START 27
#define BUTTON_STOP  26

// Relay aktif LOW
#define RELAY_PIN 2

// Waktu & logika hitung mundur
int countdownTime = 0;
bool isCounting = false;
unsigned long previousSecondMillis = 0;
unsigned long firebaseMillis = 0;

float lastLuxSent = -1;

void setup() {
  Serial.begin(115200);

  pinMode(BUTTON_10MIN, INPUT_PULLUP);
  pinMode(BUTTON_15MIN, INPUT_PULLUP);
  pinMode(BUTTON_20MIN, INPUT_PULLUP);
  pinMode(BUTTON_START, INPUT_PULLUP);
  pinMode(BUTTON_STOP, INPUT_PULLUP);
  pinMode(RELAY_PIN, OUTPUT);
  digitalWrite(RELAY_PIN, HIGH); // Relay off awal

  Wire.begin(21, 22);         // I2C utama untuk LCD
  Wire1.begin(18, 19);        // I2C kedua untuk BH1750

  lcd.init();
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print("FOTOTHERAPHY");
  delay(2000);
  lcd.clear();

  if (lightMeter.begin(BH1750::CONTINUOUS_HIGH_RES_MODE, 0x23, &Wire1)) {
    Serial.println("BH1750 siap");
  } else {
    Serial.println("Gagal inisialisasi BH1750!");
  }

  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  Serial.print("Menghubungkan WiFi");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(300);
  }
  Serial.println("\nWiFi terhubung");
  Serial.println(WiFi.localIP());

  config.api_key = API_KEY;
  config.database_url = DATABASE_URL;

  if (Firebase.signUp(&config, &auth, "", "")) {
    Serial.println("Firebase siap");
  } else {
    Serial.printf("Gagal Firebase: %s\n", config.signer.signupError.message.c_str());
  }

  Firebase.begin(&config, &auth);
  Firebase.reconnectWiFi(true);
}

String cleanString(String str) {
  str.trim();
  if (str.startsWith("\"") && str.endsWith("\"")) {
    str = str.substring(1, str.length() - 1);
  }
  return str;
}

void checkFirebaseCommand() {
  if (!Firebase.ready()) return;

  if (Firebase.RTDB.getString(&fbdo, "/Sensor/R")) {
    String cmdR = cleanString(fbdo.stringData());
    if (cmdR != "" && cmdR != "0") {
      if (cmdR == "1") {
        countdownTime = 600;
        Serial.println("Firebase: 10 menit");
      } else if (cmdR == "2") {
        countdownTime = 900;
        Serial.println("Firebase: 15 menit");
      } else if (cmdR == "3") {
        countdownTime = 1200;
        Serial.println("Firebase: 20 menit");
      }
    }
  }

  if (Firebase.RTDB.getString(&fbdo, "/Sensor/S")) {
    String cmdS = cleanString(fbdo.stringData());
    if (cmdS != "") {
      if (cmdS == "4" && countdownTime > 0 && !isCounting) {
        isCounting = true;
        previousSecondMillis = millis();
        firebaseMillis = millis();
        digitalWrite(RELAY_PIN, LOW);
        Serial.println("Firebase: Mulai");
      } else if (cmdS == "5") {
        isCounting = false;
        countdownTime = 0;
        digitalWrite(RELAY_PIN, HIGH);
        Serial.println("Firebase: Stop");
      }
    }
  }
}

void checkButtons() {
  if (digitalRead(BUTTON_10MIN) == LOW) {
    countdownTime = 600;
    Firebase.RTDB.setString(&fbdo, "/Sensor/T", "1");
    delay(80);
    Firebase.RTDB.setString(&fbdo, "/Sensor/S", " ");
    Serial.println("Tombol: 10 menit");
    delay(100);
  } else if (digitalRead(BUTTON_15MIN) == LOW) {
    countdownTime = 900;
    Firebase.RTDB.setString(&fbdo, "/Sensor/T", "2");
    delay(80);
    Firebase.RTDB.setString(&fbdo, "/Sensor/S", " ");
    Serial.println("Tombol: 15 menit");
    delay(100);
  } else if (digitalRead(BUTTON_20MIN) == LOW) {
    countdownTime = 1200;
    Firebase.RTDB.setString(&fbdo, "/Sensor/T", "3");
    delay(80);
    Firebase.RTDB.setString(&fbdo, "/Sensor/S", " ");
    Serial.println("Tombol: 20 menit");
    delay(100);
  }

  if (digitalRead(BUTTON_START) == LOW && countdownTime > 0 && !isCounting) {
    isCounting = true;
    previousSecondMillis = millis();
    firebaseMillis = millis();
    digitalWrite(RELAY_PIN, LOW);
    Firebase.RTDB.setString(&fbdo, "/Sensor/P", "1");
    Serial.println("Tombol: Mulai");
    delay(100);
  }

  if (digitalRead(BUTTON_STOP) == LOW) {
    isCounting = false;
    countdownTime = 0;
    digitalWrite(RELAY_PIN, HIGH);
    Firebase.RTDB.setString(&fbdo, "/Sensor/P", "0");
    Serial.println("Tombol: Stop");
    delay(100);
  }
}

void updateCountdown() {
  unsigned long currentMillis = millis();
  float lux = lightMeter.readLightLevel();

  // Hitung mundur setiap 1 detik
  if (isCounting && currentMillis - previousSecondMillis >= 1000) {
    previousSecondMillis += 1000;
    countdownTime--;
    if (countdownTime <= 0) {
      isCounting = false;
      countdownTime = 0;
      digitalWrite(RELAY_PIN, HIGH);
      Firebase.RTDB.setString(&fbdo, "/Sensor/S", " ");
      Firebase.RTDB.setString(&fbdo, "/Sensor/R", " ");
      Serial.println("Timer selesai");
    }
  }

  // Kirim lux ke Firebase setiap 5 detik jika berubah
  if (Firebase.ready() && currentMillis - firebaseMillis >= 5000) {
    firebaseMillis = currentMillis;
    if (lux >= 0 && abs(lux - lastLuxSent) > 5) {
      lastLuxSent = lux;
      FirebaseJson json;
      json.set("Lux", lux);
      Firebase.RTDB.updateNode(&fbdo, "/Sensor", &json);
      Serial.print("Lux dikirim: ");
      Serial.println(lux);
    }
  }

  // Update LCD
  int m = countdownTime / 60;
  int d = countdownTime % 60;
  lcd.setCursor(0, 0);
  lcd.print("Waktu = ");
  if (countdownTime > 0) {
    lcd.printf("%02d:%02d  ", m, d);
  } else {
    lcd.print("00:00    ");
  }

  lcd.setCursor(0, 1);
  lcd.print("Lux   = ");
  lcd.print((int)lux);
  lcd.print("    ");
}

void loop() {
  static unsigned long lastCmdCheck = 0;
  if (millis() - lastCmdCheck >= 100) {
    checkFirebaseCommand();
    lastCmdCheck = millis();
  }

  checkButtons();
  updateCountdown();
  delay(5);
}
